<?php

namespace Drupal\hubspot_api_webform\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission remote post handler.
 *
 * @WebformHandler(
 *   id = "hubspot_api_webform_handler",
 *   label = @Translation("HubSpot Webform Handler (hubspot_api)"),
 *   category = @Translation("External"),
 *   description = @Translation("Posts webform submissions to a Hubspot
 *   form."),
 *   cardinality =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results =
 *   \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class HubspotWebformHandler extends WebformHandlerBase {

  /**
   * The Hubspot api manager.
   *
   * @var \Drupal\hubspot_api\Manager
   */
  protected $hubspotManager;

  /**
   * The Hubspot api manager.
   *
   * @var \Drupal\hubspot_api_webform\HubspotFormManager
   */
  protected $hubspotFormManager;

  /**
   * The node storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $nodeStorage;

  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->nodeStorage = $container->get('entity_type.manager')->getStorage('node');
    $instance->hubspotManager = $container->get('hubspot_api.manager');
    $instance->hubspotFormManager = $container->get('hubspot_api_webform.form_manager');
    $instance->loggerFactory = $container->get('logger.factory');
    $instance->configFactory = $container->get('config.factory');
    $instance->conditionsValidator = $container->get('webform_submission.conditions_validator');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'hubspot_form' => NULL,
      'hubspot_mapping' => [],
      'hubspot_portal_id' => NULL,
      'static_field_mappings' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['hubspot_portal_id'] = $form_state->getValue('hubspot_portal_id');
    $guid = $this->configuration['hubspot_form'] = $form_state->getValue('hubspot_form');
    $mapping = $form_state->getValue($guid);
    $this->configuration['hubspot_mapping'] = $mapping;
    $this->configuration['static_field_mappings'] = $form_state->getValue('static_field_mappings');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $hubspot_handler = $this->hubspotManager->getHandler();
    $hubspot_forms = $hubspot_handler->forms()->all();
    $hubspot_form_options = [];
    foreach ($hubspot_forms->toArray() as $hubspot_form) {
      $hubspot_form_fields = $hubspot_handler
        ->forms()
        ->getFields($hubspot_form['guid'])
        ->getData();
      $hubspot_form_options[$hubspot_form['guid']] = $hubspot_form['name'];
      $hubspot_field_options[$hubspot_form['guid']]['fields']['--donotmap--'] = "Do Not Map";
      foreach ($hubspot_form_fields as $hubspot_form_field) {
        $hubspot_field_options[$hubspot_form['guid']]['fields'][$hubspot_form_field->name] = ($hubspot_form_field->label ? $hubspot_form_field->label : $hubspot_form_field->name) . ' (' . $hubspot_form_field->fieldType . ')';
      }

      // Collect legal consent options.
      foreach ($hubspot_form['metaData'] as $metadata) {
        if ($metadata['name'] == 'legalConsentOptions') {
          $consent_options = json_decode($metadata['value']);
          foreach ($consent_options->communicationConsentCheckboxes as $checkbox) {
            $hubspot_field_options[$hubspot_form['guid']]['fields']['consent:' . $checkbox->communicationTypeId] = 'Consent: ' . $checkbox->label . ' (' . $checkbox->communicationTypeId . ')';
          }
          break;
        }
      }
    }
    $form['hubspot_form'] = [
      '#title' => $this->t('HubSpot form'),
      '#type' => 'select',
      '#options' => $hubspot_form_options,
      '#default_value' => $this->configuration['hubspot_form'],
      '#weight' => -10,
    ];
    $form['hubspot_portal_id'] = [
      '#title' => $this->t('HubSpot Portal ID'),
      '#type' => 'textfield',
      '#default_value' => $this->configuration['hubspot_portal_id'],
      '#weight' => -9,
      '#required' => TRUE,
    ];

    foreach ($hubspot_form_options as $key => $value) {
      if ($key != '--donotmap--') {
        $form[$key] = [
          '#title' => $this->t('Field mappings for @field', [
            '@field' => $value,
          ]),
          '#type' => 'details',
          '#tree' => TRUE,
          '#weight' => 0,
          '#states' => [
            'visible' => [
              ':input[name="settings[hubspot_form]"]' =>
                [
                  'value' => $key,
                ],
            ],
          ],
        ];

        $webform = $this->getWebform()->getElementsDecodedAndFlattened();
        foreach ($webform as $form_key => $component) {
          if ($component['#type'] !== 'markup' && $component['#type'] !== 'processed_text' && $component['#type'] !== 'webform_flexbox') {
            if (isset($component['#type']) && $component['#type'] === 'webform_address' || $component['#type'] === 'address') {
              // Advanced address is 'address', Basic address is 'webform_address'. Their field codes differ.
              if ($component['#type'] === 'address') {
                $address_field_keys = ['given_name', 'family_name', 'address_line1', 'address_line2', 'locality', 'postal_code', 'country_code', 'organization'];
              }
              else {
                // Basic address fields.
                $address_field_keys = ['address', 'address_2', 'city', 'state_province', 'postal_code', 'country'];
              }
              foreach ($address_field_keys as $sub_key) {
                $subcomponent_key = $form_key. '___' . $sub_key;
                $default_value = NULL;
                if ($this->configuration['hubspot_form'] == $key) {
                  $default_value = isset($this->configuration['hubspot_mapping'][$subcomponent_key]) ? $this->configuration['hubspot_mapping'][$subcomponent_key] : NULL;
                }
                $form[$key][$subcomponent_key] = [
                  '#title' => (isset($component['#title']) ? $component['#title'] : '') . '[' . $sub_key . '] (' . $component['#type'] . ')',
                  '#type' => 'select',
                  '#options' => $hubspot_field_options[$key]['fields'],
                  '#default_value' => $default_value,
                ];
              }
            }
            $default_value = NULL;
            if ($this->configuration['hubspot_form'] == $key) {
              $default_value = isset($this->configuration['hubspot_mapping'][$form_key]) ? $this->configuration['hubspot_mapping'][$form_key] : NULL;
            }
            $form[$key][$form_key] = [
              '#title' => (isset($component['#title']) ? $component['#title'] : '') . ' (' . $component['#type'] . ')',
              '#type' => 'select',
              '#options' => $hubspot_field_options[$key]['fields'],
              '#default_value' => $default_value,
            ];
          }
        }
      }
    }

    $form['static_field_mappings'] = [
      '#title' => $this->t('Static field mappings'),
      '#type' => 'textarea',
      '#default_value' => $this->configuration['static_field_mappings'],
      '#description' => $this->t('Configuration for static field mappings. Enter one field mapping per line. Format: HUBSPOT_FIELD_NAME|VALUE'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE, $json_fields = []) {
    $operation = ($update) ? 'update' : 'insert';
    $post_data = $this->getPostData($operation, $webform_submission);
    $entity_type = isset($post_data['entity_type']) ? $post_data['entity_type'] : NULL;
    $entity_id = isset($post_data['entity_id']) ? $post_data['entity_id'] : NULL;
    $url = '';
    $title = '';
    try {
      $entity = \Drupal::entityTypeManager()
        ->getStorage($entity_type)
        ->load($entity_id);
      $url = $entity->toUrl('canonical', ['absolute' => TRUE])->toString();
      $title = $entity->label();
    } catch (\Exception $e) {
      // DO NOTHING FOR NOW.
    }
    $form_guid = $this->configuration['hubspot_form'];
    $portal_id = $this->configuration['hubspot_portal_id'];;
    $fields = $this->configuration['hubspot_mapping'];

    $webform_elements = $this->getWebform()->getElementsDecodedAndFlattened();
    // Configured field mappings.
    foreach ($fields as $webform_name => $hubspot_name) {
      if ($hubspot_name !== '--donotmap--') {
        if (isset($post_data[$webform_name]) && is_array($post_data[$webform_name])) {
          $post_data[$webform_name] = join(";", $post_data[$webform_name]);
        }
        // Handle sub element like
        if (strpos($webform_name, '___') !== FALSE) {
          $keys = explode('___', $webform_name);
          if (isset($post_data[$keys[0]][$keys[1]])) {
            $json_fields[$hubspot_name] =  $post_data[$keys[0]][$keys[1]];
          }
        }
        else if (strpos($hubspot_name, 'consent:') === 0) {
          if ($post_data[$webform_name]) {
            // Add consent if checked with field title (consent text).
            $json_fields[$hubspot_name] = $webform_elements[$webform_name]['#title'];
          }
        }
        else if (isset($post_data[$webform_name])) {
          $json_fields[$hubspot_name] = $post_data[$webform_name];
        }
      }
    }

    // Add static field mappings.
    $static_fields = explode("\n", $this->configuration['static_field_mappings']);
    foreach ($static_fields as $field) {
      $value = explode('|', $field, 2);
      if (count($value) === 2 && !isset($json_fields[$value[0]])) {
        $json_fields[$value[0]] = trim($value[1]);
      }
    }

    $this->hubspotFormManager->submit($portal_id, $form_guid, $json_fields, $url, $title);
  }

  /**
   * Get a webform submission's post data.
   *
   * @param string $operation
   *   The type of webform submission operation to be posted. Can be 'insert',
   *   'update', or 'delete'.
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   The webform submission to be posted.
   *
   * @return array
   *   A webform submission converted to an associative array.
   */
  protected function getPostData($operation, WebformSubmissionInterface $webform_submission) {
    // Get submission and elements data.
    $data = $webform_submission->toArray(TRUE);

    // Flatten data.
    // Prioritizing elements before the submissions fields.
    $data = $data['data'] + $data;
    unset($data['data']);

    return $data;
  }

}
